#include "cam.h"

#include "vmath.h"

#define	ANGLE_MASK	((TWOPI)-1)

static dvec3 pos;
static dvec3 deffwd={0,0,1};
static dvec3 defup={0,1,0};
static dvec3 forward,up;
static dmatr3 rotmatr,tm;

static int xa,ya,za;

static void mulrotmatr(void) {	/* Multiply rotmatr by tm */
   double sum;
   int i,j,k;
   dmatr3 r;
   for(j=0;j<N_DIMS;j++) {
      for(i=0;i<N_DIMS;i++) {
         sum=0;
         for(k=0;k<N_DIMS;k++)
            sum+=rotmatr[j][k]*tm[k][i];
         r[j][i]=sum;
      }
   }
   for(j=0;j<N_DIMS;j++) {
      for(i=0;i<N_DIMS;i++) rotmatr[j][i]=r[j][i];
   }
} /* mulrotmatr */
static void clrmatr(dmatr3 *p) {
   int j,i;
   for(j=0;j<N_DIMS;j++) for(i=0;i<N_DIMS;i++) (*p)[j][i]=0;
} /* clrmatr */
static double _arg(int a) { return (a*M_PI)*2.0/TWOPI; }
static double _sin(int a) {
   return sin(_arg(a));
} /* _sin */
static double _cos(int a) {
   return cos(_arg(a));
} /* _cos */
static void apply_tm(int c1,int c2,int angle) {
   double s=_sin(angle),c=_cos(angle);
   int i;
   clrmatr(&tm);
   for(i=0;i<N_DIMS;i++) tm[i][i]=1;
   tm[c1][c1]=tm[c2][c2]=c;
   tm[c1][c2]=-(tm[c2][c1]=s);
   mulrotmatr();
} /* init_tm */
static void apply_rotm(dvec3*vec,dvec3*dest) {
   int j,i;
   double s;
   for(j=0;j<N_DIMS;j++) {
      s=0;
      for(i=0;i<N_DIMS;i++) s+=(rotmatr[j][i])*(*vec)[i];
      (*dest)[j]=s;
   }
} /* apply_rotm */
static void mk_rotmatr(void) {
   int i,j;
   double s,c;
   xa&=ANGLE_MASK; ya&=ANGLE_MASK; za&=ANGLE_MASK;
   clrmatr(&rotmatr);
   for(i=0;i<N_DIMS;i++) rotmatr[i][i]=1;
   clrmatr(&tm);
   tm[0][0]=1.0;	/* Around Ox - touch y and z */
   apply_tm(X_DIM,Z_DIM,za);
   apply_tm(X_DIM,Y_DIM,ya);
   apply_tm(Y_DIM,Z_DIM,xa);
   apply_rotm(&deffwd,&forward);
   apply_rotm(&defup,&up);
} /* mk_rotmatr */

void cam_init(double x,double y,double z) {
   pos[0]=x; pos[1]=y; pos[2]=z;
   xa=ya=za=0;
   cam_rotate(0,0,0);
} /* cam_init */

void cam_rotate(int dxa,int dya,int dza) {
   xa+=dxa; ya+=dya; za+=dza;
   dxa&=ANGLE_MASK; dya&=ANGLE_MASK; dza&=ANGLE_MASK;
   mk_rotmatr();
} /* cam_rotate */

void cam_move(double speed) {
   int i;
   for(i=0;i<N_DIMS;i++) pos[i]+=speed*forward[i];
} /* cam_move */

void cam_getview(dvec3 *_pos,dvec3*_fwd,dvec3*_up) {
   int i;
   for(i=0;i<N_DIMS;i++) {
      (*_pos)[i]=pos[i];
      (*_fwd)[i]=forward[i];
      (*_up)[i]=up[i];
   }
} /* cam_getview */
