#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "gr.h"

#include "cam.h"
#include "vox.h"

#include "obj.h"

#define	RESOLUTION	1

#define	NC	400
#define	MAXC	20
#define	TMAX	TRANSPFULL/4
#define	NT	50

#define	SQSZ	11
#define	SQH	1
#define	N	16

int width=0,height=0,mode=-1;
#pragma	argsused
static int enumerator(int wid,int hig,int rbits,int gbits,int bbits,
		       int pages,int code) {
   int remember=0;
   if(width<wid) {
      remember=1;
   } else if(width==wid) { remember=1;}
   if(remember) {
      width=wid; height=hig; mode=code;
   }
   return 1;
} /* enumerator */

static void chk_created(int r) {
   if(!r) { printf("Error creating one of test objects!\n"); exit(1);}
} /* chk_created */

static colorinfo purered={MAXCOLORCOMP,0,0},
                 puregreen={0,MAXCOLORCOMP,0},
                 pureblue={0,0,MAXCOLORCOMP},
                 purewhite={MAXCOLORCOMP,MAXCOLORCOMP,MAXCOLORCOMP};
static colorinfo dirtbrown={MAXCOLORCOMP/2,MAXCOLORCOMP/2,0},
                 darkbrown={MAXCOLORCOMP/3,MAXCOLORCOMP/4,0},
                 darkred={MAXCOLORCOMP*3/7,0,0},
                 yellow={MAXCOLORCOMP,MAXCOLORCOMP,0};
void make_square(longint x,longint z,int cidx) {
   colorinfo *color;
   if(cidx&1) color=&purered; else color=&purewhite;
   obj_setcoord(x,50,z);
   obj_setsize(SQSZ,SQH,SQSZ); obj_setisocolor(color,TRANSPNONE);
   chk_created(create_bar());
} /* make_square */
void make_chessboard(void) {
   int i,j;
   longint x=-SQSZ*N/2,z=-SQSZ*N/2;
   for(j=0;j<N;j++)
      for(i=0;i<N;i++)
         make_square(x+i*SQSZ,z+j*SQSZ,i^j);
} /* make_chessboard */
void add_test_voxels(void) {
   make_chessboard();
   oct_update();
} /* add_test_voxels */
void draw_test_scene(void) {
   dvec3 pos,fwd,up;
   cam_getview(&pos,&fwd,&up);
   vox_view(&pos,&fwd,&up);
   vox_reset();
   ov_subdivide(&fwd,RESOLUTION);
   vox_draw();
   cprintf("%lf,%lf,%lf\r",pos[0],pos[1],pos[2]);
   gr_redraw();
} /* draw_test_scene */

#define	ROTSPEED	10
void test_drawing(void) {
   int done=0,k,speed=0;
   while(!done) {
      cam_move(speed);
      draw_test_scene();
      do {
         k=gr_key();
         switch(k) {
            case 0x1b:
            case '`': case '~': done=1; break;
            case '+': speed++; break;
            case '-': speed--; break;
            case '/': speed=0; break;
            case 's': case 'S': cam_rotate(0,0, ROTSPEED); break;
            case 'f': case 'F': cam_rotate(0,0,-ROTSPEED); break;
            case 'e': case 'E': cam_rotate( ROTSPEED,0,0); break;
            case 'd': case 'D': cam_rotate(-ROTSPEED,0,0); break;
            case 'O': case 'o': oct_optimize(); printf("Optimized!\n"); break;
	    case 'p': case 'P': if(!gr_shot()) { printf("\b\b\b"); } break;
	    default: break;
         }
      } while(k);
   }
} /* test_drawing */

void main(void) {
   gr_enummodes(&enumerator);
   if(mode<0) { printf("No graphic modes!\n"); exit(1); }
   if(!vox_init(120,width,height)) { printf("Error initializing renderer!\n"); exit(1);}
   if(!oct_init(4000)) { printf("Error initializing octree!\n"); exit(1);}
   add_test_voxels();
   cam_init(-110,0,-150);
   cam_rotate(ROTSPEED*0,0,-ROTSPEED*10);
   if(!gr_open(mode)) { printf("Error initializing graphics!\n"); exit(1); }
   test_drawing();
   gr_close();
   oct_done();
   vox_done();
   exit(0);
} /* main */
