#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "gr.h"

#include "cam.h"
#include "vox.h"

#include "obj.h"

#define	RESOLUTION	1

#define	NC	400
#define	MAXC	20
#define	TMAX	TRANSPFULL/4
#define	NT	50

int width=0,height=0,mode=-1;
#pragma	argsused
static int enumerator(int wid,int hig,int rbits,int gbits,int bbits,
		       int pages,int code) {
   int remember=0;
   if(width<wid) {
      remember=1;
   } else if(width==wid) { remember=1;}
   if(remember) {
      width=wid; height=hig; mode=code;
   }
   return 1;
} /* enumerator */

static void chk_created(int r) {
   if(!r) { printf("Error creating one of test objects!\n"); exit(1);}
} /* chk_created */

static colorinfo purered={MAXCOLORCOMP,0,0},
                 puregreen={0,MAXCOLORCOMP,0},
                 pureblue={0,0,MAXCOLORCOMP},
                 purewhite={MAXCOLORCOMP,MAXCOLORCOMP,MAXCOLORCOMP};
static colorinfo dirtbrown={MAXCOLORCOMP/2,MAXCOLORCOMP/2,0},
                 darkbrown={MAXCOLORCOMP/3,MAXCOLORCOMP/4,0},
                 darkred={MAXCOLORCOMP*3/7,0,0},
                 yellow={MAXCOLORCOMP,MAXCOLORCOMP,0};
void make_floor(void) {
   obj_setisocolor(&darkred,TRANSPNONE);
   obj_setcoord(-100,20,-100);
   obj_setsize(200,2,200);
   chk_created(create_bar());
} /* make_floor */
void make_fence(void) {
   int i;
   obj_setisocolor(&dirtbrown,TRANSPNONE);
   for(i=0;i<30;i++) {
      obj_setcoord(-100+i*5,10,-90);
      obj_setsize(1,10,1);
      chk_created(create_bar());
   }
   obj_setcoord(-105,11,-90); obj_setsize(160,1,1);
   chk_created(create_bar());
} /* make_fence */
static colorinfo lightcolor={MAXSTOREDCOLORCOMP,
                             MAXSTOREDCOLORCOMP,
                             MAXSTOREDCOLORCOMP/8};
void make_light(void) {
   obj_setisocolor(&lightcolor,TRANSPNONE);
   obj_setcoord(-30,-0,100); obj_setsize(3,0,0);
   chk_created(create_sphere());
} /* make_light */
static colorinfo greycolor={MAXCOLORCOMP/2,MAXCOLORCOMP/2,MAXCOLORCOMP/2};
static colorinfo grasscolor={0,MAXCOLORCOMP/2,0};
#define	GHEIGHT	40
#define	DISPOS	(GHEIGHT*30/100)
void make_grass(void) {
   longint x,z;
   for(z=-30;z<30;z++) {
      for(x=-80;x<20;x++) {
         if(random(10000)>9880) {
            chk_created(line3d(x,20,z,
                               x+random(DISPOS)-DISPOS/2,-20,z+random(DISPOS)-DISPOS/2,
                               1,&grasscolor,TRANSPNONE)
                        );
         }
      }
   }
} /* make_grass */
void add_test_voxels(void) {
   make_fence();
   make_floor();
   make_light();
   make_grass();
   oct_update();
//oct_optimize();
} /* add_test_voxels */
void draw_test_scene(void) {
   dvec3 pos,fwd,up;
   cam_getview(&pos,&fwd,&up);
   vox_view(&pos,&fwd,&up);
   vox_reset();
   ov_subdivide(&fwd,RESOLUTION);
   vox_draw();
   cprintf("%lf,%lf,%lf\r",pos[0],pos[1],pos[2]);
   gr_redraw();
} /* draw_test_scene */

#define	ROTSPEED	10
void test_drawing(void) {
   int done=0,k,speed=0;
   while(!done) {
      cam_move(speed);
      draw_test_scene();
      do {
         k=gr_key();
         switch(k) {
            case 0x1b:
            case '`': case '~': done=1; break;
            case '+': speed++; break;
            case '-': speed--; break;
            case '/': speed=0; break;
            case 's': case 'S': cam_rotate(0,0, ROTSPEED); break;
            case 'f': case 'F': cam_rotate(0,0,-ROTSPEED); break;
            case 'e': case 'E': cam_rotate( ROTSPEED,0,0); break;
            case 'd': case 'D': cam_rotate(-ROTSPEED,0,0); break;
            case 'O': case 'o': oct_optimize(); printf("Optimized!\n"); break;
	    case 'p': case 'P': if(!gr_shot()) { printf("\b\b\b"); } break;
	    default: break;
         }
      } while(k);
   }
} /* test_drawing */

void main(void) {
   gr_enummodes(&enumerator);
   if(mode<0) { printf("No graphic modes!\n"); exit(1); }
   if(!vox_init(90,width,height)) { printf("Error initializing renderer!\n"); exit(1);}
   if(!oct_init(4000)) { printf("Error initializing octree!\n"); exit(1);}
   add_test_voxels();
   cam_init(0,-50,-250);
   cam_rotate(ROTSPEED*0,0,0);
   if(!gr_open(mode)) { printf("Error initializing graphics!\n"); exit(1); }
   test_drawing();
   gr_close();
   oct_done();
   vox_done();
   exit(0);
} /* main */
